From 63942679046c9d5a2942c257651270090d6b30d5 Mon Sep 17 00:00:00 2001
From: Pavel Shramov <shramov@mexmat.net>
Date: Mon, 4 Oct 2010 00:15:06 +0400
Subject: [PATCH] Added reparent

Added ability to embed into other X-window when called with -w/--window
parametere.

Signed-off-by: Pavel Shramov <shramov@mexmat.net>
---
 camview/camview.c |   28 ++++++++++++++++++++++++++--
 1 files changed, 26 insertions(+), 2 deletions(-)

diff --git a/camview/camview.c b/camview/camview.c
index e479512..228dd53 100644
--- a/camview/camview.c
+++ b/camview/camview.c
@@ -7,6 +7,8 @@
 #include <errno.h>
 
 #include <gtk/gtk.h>
+#include <gdk/gdkx.h>
+#include <X11/Xlib.h>
 
 #include <camunits/cam.h>
 #include <camunits-gtk/cam-gtk.h>
@@ -25,6 +27,7 @@ typedef struct _state_t {
     char *xml_fname;
     char *extra_plugin_path;
     int use_gui;
+    int window_id;
 
     GtkWindow *window;
     GtkWidget *manager_frame;
@@ -194,7 +197,10 @@ on_save_menu_item_activate (GtkWidget *widget, void * user)
 static void
 setup_gtk (state_t *self)
 {
-    self->window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
+    GtkWindowType type = GTK_WINDOW_TOPLEVEL;
+    if (self->window_id)
+        type = GTK_WINDOW_POPUP;
+    self->window = GTK_WINDOW (gtk_window_new (type));
     gtk_window_set_title(self->window, "Camview");
     gtk_window_set_resizable(self->window, TRUE);
     gtk_window_set_default_size(self->window, 1000, 540);
@@ -316,6 +322,19 @@ setup_gtk (state_t *self)
 
     gtk_widget_show_all (GTK_WIDGET (self->window));
 
+    if (self->window_id) {
+        Window xid = GDK_WINDOW_XID (GTK_WIDGET(self->window)->window);
+        long info[2] = { 0, 1 << 0 /*XEMBED_MAPPED*/ };
+        Display * dpy = GDK_WINDOW_XDISPLAY(GTK_WIDGET(self->window)->window);
+        Atom atom = XInternAtom(dpy, "_XEMBED_INFO", 1);
+        printf("Reparent %d (%p)\n", xid, dpy);
+        XChangeProperty(dpy, xid, atom, atom, 32, PropModeReplace, (unsigned char *)&info, 2);
+        XReparentWindow(dpy, xid, self->window_id, 0, 0);
+	XSync(dpy, 0);
+	XMapWindow(dpy, xid);
+	XSync(dpy, 0);
+    }
+
     g_timeout_add(250, decay_fps, self);
 }
 
@@ -390,6 +409,7 @@ usage()
     "chains.  It uses GTK+ 2.0, OpenGL, and libcamunits.\n"
     "\n"
     "Options:\n"
+    "  -w, --window WID     Embed in window with id WID\n"
     "  -c, --chain NAME     Load chain from file NAME\n"
     "  --no-gui             Run without a GUI.  If --no-gui is specified,\n"
     "                       then -c is required.\n"
@@ -405,11 +425,12 @@ int main (int argc, char **argv)
     state_t * self = (state_t*) calloc (1, sizeof (state_t));
     self->use_gui = 1;
 
-    char *optstring = "hc:p:";
+    char *optstring = "hc:p:w:";
     int c;
     struct option long_opts[] = { 
         { "help", no_argument, 0, 'h' },
         { "chain", required_argument, 0, 'c' },
+        { "window", required_argument, 0, 'w' },
         { "plugin-path", required_argument, 0, 'p' },
         { "no-gui", no_argument, 0, 'u' },
         { 0, 0, 0, 0 }
@@ -418,6 +439,9 @@ int main (int argc, char **argv)
     while ((c = getopt_long (argc, argv, optstring, long_opts, 0)) >= 0)
     {
         switch (c) {
+            case 'w':
+                self->window_id = atoi (optarg);
+                break;
             case 'c':
                 self->xml_fname = strdup (optarg);
                 break;
-- 
1.7.1

